// blood PDF by gnuplot
// Using gnuplot script topdf.plt
package main

import (
	"encoding/csv"
	"flag"
	"fmt"
	"io/ioutil"
	"os"
	"os/exec"
	"strconv"
	"bufio"
)

const dbfile = "current.csv"

type AAy [][4]int // main data type
var bld AAy       // blood data

const ( // rows index name 0 .. 3
	ymdh = iota
	hi
	low
	pls
)

func failOnError(err error) {
	if err != nil {
		panic(err)
	}
}

func ag(datafile string) string {     // for gnuplot option
        fd, err := os.Open(datafile)
        failOnError(err)
        defer fd.Close()
	r := bufio.NewReader(fd)
	line, err := r.ReadString('\n')
	failOnError(err)
	return fmt.Sprintf("when=%s", line[:6])
}

func mycsv(csvfile string) {
	var rows [4]int
	fd, err := os.Open(csvfile)
	failOnError(err)
	defer fd.Close()
	reader := csv.NewReader(fd)
	all, err := reader.ReadAll()
	failOnError(err)
	for _, el := range all {
		for i, v := range el {
			rows[i], err = strconv.Atoi(v)
			failOnError(err)
		}
		bld = append(bld, rows)
	}
}

func savecsv(csf string) {
	buf, _ := ioutil.ReadFile(csf)
	_ = ioutil.WriteFile("Backup.csv", buf, 0666)
	fd, err := os.OpenFile(csf, os.O_CREATE|os.O_WRONLY, 0666)
	failOnError(err)
	err = fd.Truncate(0)
	failOnError(err)
	defer func() {
		fd.Close()
	}()
	w := csv.NewWriter(fd)
	for _, el := range bld {
		rows := make([]string, 0)
		for _, v := range el {
			rows = append(rows, strconv.Itoa(v))
		}
		w.Write(rows)
	}
	w.Flush()
}

func ire(ym int) {
	var a, b, c, d string
	var rs [4]int
	for {
		a, b, c, d = "", "", "", ""
		fmt.Printf("%d> ", ym)
		fmt.Scanln(&a, &b, &c, &d)
		if a == "fin" {
			break
		}
		rs[ymdh], _ = strconv.Atoi(a)
		rs[hi], _ = strconv.Atoi(b)
		rs[low], _ = strconv.Atoi(c)
		rs[pls], _ = strconv.Atoi(d)
                if rs[ymdh] < 100 || rs[ymdh] > 3123 || (rs[ymdh]%100) > 23 ||
                     rs[hi] < 80 || rs[low] < 40 || rs[pls] < 40 {
			fmt.Println("Bad data")
			continue
		}
		rs[ymdh] += (ym * 10000)
		if rs[ymdh] <= bld[len(bld)-1][ymdh] {
			fmt.Println("Bad seq.")
			continue
		}
		bld = append(bld, rs)
	}
	savecsv(dbfile)
}

func (ds AAy) pp(fn string) {
	xf,_ := os.Create(fn)
        defer xf.Close()
	for _, el := range ds {
		fmt.Fprintf(xf, "%d %d %d\n", el[ymdh], el[hi], el[low])
	}
}

func (ds AAy) hd(n int) AAy {
	if len(ds) < n {
		fmt.Println("Warnning: Req size is to big, apply possible size.")
		return ds
	}
	return ds[:n]
}

func (ds AAy) tl(n int) AAy {
	if len(ds) < n {
		fmt.Println("Warnning: Req size is to big, apply possible size.")
		return ds
	}
	return ds[len(ds)-n:]
}

func (ds AAy) am() AAy {
	var rs AAy
	for _, el := range ds {
		if el[ymdh]%100 < 12 {
			rs = append(rs, el)
		}
	}
	return rs
}

func (ds AAy) pm() AAy {
	var rs AAy
	for _, el := range ds {
		if el[ymdh]%100 >= 12 {
			rs = append(rs, el)
		}
	}
	return rs
}

func (ds AAy) frm(ym int) AAy {
	var rs AAy
	lmt := ym * 10000
	if ym > 9999 {            // Case yymmdd
		lmt = ym * 100
	}
	for _, el := range ds {
		if el[ymdh] > lmt {
			rs = append(rs, el)
		}
	}
	return rs
}

func (ds AAy) utl(ym int) AAy {
	var rs AAy
	lmt := ym*10000 + 3124
	for _, el := range ds {
		if el[ymdh] < lmt {
			rs = append(rs, el)
		}
	}
	return rs
}

func (ds AAy) eq(ym int) AAy {
	var rs AAy
	for _, el := range ds {
		if el[ymdh]/10000 == ym {
			rs = append(rs, el)
		}
	}
	return rs
}

func mksf(sf string, gsrc string){
        fd, err := os.OpenFile(sf, os.O_RDWR|os.O_CREATE, 0755)
        failOnError(err)
        defer fd.Close()
        fd.WriteString(gsrc)
}

func main() {
gsrc :=`# blood graph for gnuplot
set encoding utf8
set terminal pdfcairo mono font ",20" size 21cm, 29cm
set output "./zzz.pdf"
#set output "Desktop\\" . when . ".pdf"

stats "am.dat" using 2:3
amh = sprintf(" 最高血圧(平均=%.1f 偏差=%.1f)", STATS_mean_x, STATS_stddev_x)
aml = sprintf(" 最低血圧(平均=%.1f 偏差=%.1f)", STATS_mean_y, STATS_stddev_y)
stats "pm.dat" using 2:3
pmh = sprintf(" 最高血圧(平均=%.1f 偏差=%.1f)", STATS_mean_x, STATS_stddev_x)
pml = sprintf(" 最低血圧(平均=%.1f 偏差=%.1f)", STATS_mean_y, STATS_stddev_y)

set grid
set yrange [50:160]
set ytics 10
unset key                   # no label on right top
set xdata time
set timefmt "%y%m%d%H"
set format x "%m/%d"        # m/d/y -> m/d

set multiplot layout 2,1

set title  '起床時: ' . amh . aml . when
plot "am.dat" using 1:2 with lines lt -1, "am.dat" using 1:3 with lines lt -1

set title  '就寝時: ' . pmh . pml . when
plot "pm.dat" using 1:2 with lines lt -1, "pm.dat" using 1:3 with lines lt -1

unset multiplot
set terminal dumb
## end of gnuplot-script
`
	var vire *int = flag.Int("ire", 1811, "Input YYMM's data")
	var stym *int = flag.Int("from",1302, "Make PDF at YYMM[DD]")
	flag.Parse()

	mycsv(dbfile)
	if *vire != 1811 {
		ire(*vire)
	}

	sz := 70               // 10 week's
	if *stym <= 1302 {
		bld.am().tl(sz).pp("am.dat")
		bld.pm().tl(sz).pp("pm.dat")
	} else {
		bld.am().frm(*stym).hd(sz).pp("am.dat")
		bld.pm().frm(*stym).hd(sz).pp("pm.dat")
	}

	mksf("topdf.plt", gsrc)
	exec.Command("gnuplot", "-e", ag("am.dat"), "topdf.plt").Run()
        os.Remove("am.dat")
        os.Remove("pm.dat")
	os.Remove("topdf.plt")
}
